#include <iostream>
#include <limbo/bayes_opt/boptimizer.hpp>

using namespace limbo;

struct Params {
    struct bayes_opt_boptimizer {
        BO_PARAM(double, noise, 0.0);
    };

    struct bayes_opt_bobase {
        BO_PARAM(int, stats_enabled, false);
    };

    struct init_randomsampling {
        BO_PARAM(int, samples, 10);
    };

    struct stop_maxiterations {
        BO_PARAM(int, iterations, 40);
    };

    struct acqui_gpucb : public defaults::acqui_gpucb {
    };

    struct opt_gridsearch : public defaults::opt_gridsearch {
    };

    struct opt_rprop : public defaults::opt_rprop {
    };

    struct opt_parallelrepeater : public defaults::opt_parallelrepeater {
    };
};


struct Eval {
    static constexpr size_t dim_in = 1;
    static constexpr size_t dim_out = 1;

    Eigen::VectorXd operator()(const Eigen::VectorXd& x) const
    {
        Eigen::VectorXd res(1);
        res(0) = -((5 * x(0) - 2.5) * (5 * x(0) - 2.5)) + 5;
        return res;
    }
};

int main() {
    bayes_opt::BOptimizer<Params> boptimizer;
    boptimizer.optimize(Eval());
    std::cout << "Best sample: " << boptimizer.best_sample()(0) << " - Best observation: " << boptimizer.best_observation()(0) << std::endl;
    return 0;
}
